# -*- coding: utf-8 -*-
##############################################################################
#
#    Email marketing click & read tracker
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from md5 import md5
from openerp.osv import osv, fields
from openerp.tools import config
from replace import insert_tracker_in_html, insert_tracker_in_text


class hooked_email_template(osv.Model):
    """overrides the base email template to hook the URL tracker
    """
    _inherit = 'email.template'

    def generate_email(self, cr, uid, template_id, res_id, context=None):
        """override generate_email() from our parent to substitute URLs
        """

        values = super(hooked_email_template, self).generate_email(
            cr, uid, template_id, res_id, context=context)

        if context is None:
            context = dict()

        if context.get('use_tracker', False):
            # here we need to use tracker so let's do it
            tracker_base = context.get('tracker_base')

            # here we add a hash that contains informations about our
            # business unit... this is necessary to be able to have
            # a tracker serve multiple business units
            hostname_bu = config.get('xmlrpc_interface')
            if not hostname_bu:
                hostname_bu = "localhost"
            port_bu = config.get('xmlrpc_port')
            database_bu = cr.dbname
            id_bu = "%s:%s/%s" % (hostname_bu, port_bu, database_bu)
            tracker_base += '/' + md5(id_bu).hexdigest()

            name_clicker = ""
            obj_osv = self.pool.get(values['model'])
            if obj_osv:
                obj_rec = obj_osv.browse(cr, uid,
                                         values['res_id'],
                                         context=context)
                if obj_rec:
                    name_clicker = obj_rec.name

            trackitem_osv = self.pool.get(
                'marketing_campaign_tracker.trackitem')

            new_body_text, tracks = insert_tracker_in_text(
                values.get('body'), tracker_base, context['activity_id'],
                context['campaign_id'], name_clicker)

            for trackvalues in tracks:
                # create a trackitem for each URL that is replaced
                trackitem_osv.create(cr, uid, trackvalues, context=None)

            values["body"] = new_body_text

            new_body_html, tracks = insert_tracker_in_html(
                values.get('body_html'), tracker_base, context['activity_id'],
                context['campaign_id'], name_clicker)

            for trackvalues in tracks:
                # create a trackitem for each URL that is replaced
                trackitem_osv.create(cr, uid, trackvalues, context=None)

            values["body_html"] = new_body_html

        return values


class marketing_campaign_activity(osv.Model):
    # we want to change the marketing campaign activity to inject our new
    #  options
    _inherit = 'marketing.campaign.activity'

    # here we add a new field to the marketing campaign activity
    # the use_tracker boolean will activate the tracking for URLs if is
    # set to True. The default value will be False in order to avoid any
    # problem with existing activities previous to this module installation.
    _columns = dict(
        use_tracker=fields.boolean(
            'Use Tracker',
            help='Activates the usage of an external "Bounced" click tracker'),
        tracker_base=fields.char(
            "Tracker Base URL", size=200,
            help="The base URL on which the tracker listens"),
    )

    # make sure our new field has a sensible default.
    # see comment above the _columns definition for more
    _defaults = dict(
        use_tracker=False,
        tracker_base=None,
    )

    # make sure an activity which uses a tracker always has a tracker
    # base defined...
    def _check_baseurl(self, cr, uid, ids, context=None):
        """Validates that every activity which has the use_tracker option set
        also has a tracker_base defined
        """
        activities = self.browse(cr, uid, ids, context=context)

        for activity in activities:
            if activity.use_tracker and not activity.tracker_base:
                return False

        return True

    _constraints = [
        (
            _check_baseurl,
            'If you use tracking you must give a tracker base URL',
            ['tracker_base']
        )
    ]

    # override the _process_wi_email to replace all the URLs in the template
    # body by our tracker URL and store the corresponding keys in database
    def _process_wi_email(self, cr, uid, activity, workitem, context=None):
        if context is None:
            context = dict()

        context['use_tracker'] = activity.use_tracker

        if activity.use_tracker:
            # remove any trailing slash that could be present in the base_url
            # and inject the necessary keys into our context
            context['tracker_base'] = activity.tracker_base.rstrip('/')
            context['activity_id'] = activity.id
            context['campaign_id'] = activity.campaign_id.id

        return super(marketing_campaign_activity, self)._process_wi_email(
            cr, uid, activity, workitem, context=context
        )


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
