# -*- coding: utf-8 -*-
##############################################################################
#
#    Marketing campaign analytic.
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields

class campaign_real_url(osv.Model):
    _name = 'marketing_campaign_tracker.campaign_real_url'

    _columns = {
        'analytic_id': fields.many2one(
            'marketing_campaign_tracker.analytic',
        ),
        'real_url': fields.char('Real URL', size=255),
        'clicks': fields.integer('Clicks'),
        'clicker': fields.integer('Clicker'),
        'timedate': fields.integer('Timedate'),
    }

class analytic(osv.Model):
    """Store and calculate information to display analytic
    graphs of all running marketing campaign
    """

    _name = 'marketing_campaign_tracker.analytic'

    def empty_campaign_real_url_table(self, cr, uid, context=None):
        campaign_real_url_osv = self.pool.get('marketing_campaign_tracker.campaign_real_url')
        campaign_real_url_ids = campaign_real_url_osv.search(cr, uid, [], context=context)
        campaign_real_url_osv.unlink(cr, uid, campaign_real_url_ids, context=context)

    def count_real_url(self, cr, uid, campaign_id, context=None):
        trackitem_osv = self.pool.get('marketing_campaign_tracker.trackitem')
        trackitem_ids = trackitem_osv.search(cr, uid,
                                             [('campaign_id', '=', campaign_id)],
                                             context=context)
        trackitems = trackitem_osv.browse(cr, uid, trackitem_ids, context=context)
        campaign_real_url_osv = self.pool.get('marketing_campaign_tracker.campaign_real_url')
        self.empty_campaign_real_url_table(cr, uid, context=context)
        analytic_id = self.search(cr, uid, [('campaign_id', '=', campaign_id)], context=context)
        for trackitem in trackitems:
            url_count = 0
            is_in_table = campaign_real_url_osv.search(cr, uid,
                                [('real_url', '=', trackitem.real_url)],
                                context=context)
            if not is_in_table:
                url_count = trackitem_osv.search(cr, uid,
                                [('real_url', '=', trackitem.real_url)],
                                count=True, context=context)
                campaign_real_url_osv.create(cr, uid,
                                    {'analytic_id': analytic_id[0],
                                     'real_url': trackitem.real_url,
                                     'clicks': url_count},
                                    context=context)



    def update_graphs(self, cr, uid, campaign_id, context=None):
        analytic_id = self.search(cr, uid, [('campaign_id', '=', campaign_id)], context=context)
        self.count_real_url(cr, uid, campaign_id, context=context)
        if not analytic_id:
            self.create(cr, uid, {'campaign_id': campaign_id}, context=context)
        return 0

    _columns = {
        'campaign_id': fields.integer(
            'Campaign ID'
        ),
        'urls': fields.one2many('marketing_campaign_tracker.campaign_real_url',
                                'analytic_id',
                                'URLs'),
    }

